# -*- coding:utf-8 -*-
#
# Copyright (C) 2019-2020 Alibaba Group Holding Limited


from __future__ import print_function
from .yoc import *
from .subcmds import all_commands
import optparse
import sys
import os

import warnings
warnings.filterwarnings("ignore", category=SyntaxWarning)

__version__ = "1.1.40"
__auto_init__ = 12345

global_options = optparse.OptionParser(
    usage="aos COMMAND [ARGS]",
    description="For more information about aos commands list, please see `aos help`.\t\tFor a specific command, please see `aos help <command>`."
)


class YocCommand:
    def __init__(self):
        self.conf = Configure()
        self.commands = {}
        if self.conf.init:
            self.commands = all_commands
        else:
            self.commands['init'] = all_commands['init']
            self.commands['help'] = all_commands['help']
            self.commands['toolchain'] = all_commands['toolchain']
            self.commands['create project'] = all_commands['create project']
            all_commands['help'].commands = self.commands

    def _ParseArgs(self, argv):
        """Parse the main `aos` command line options."""
        name = None
        glob = []

        for i in range(len(argv)):
            if not argv[i].startswith('-'):
                name = argv[i]
                if i > 0:
                    glob = argv[:i]
                
                if len(argv) > i+1:
                    got_flag = None
                    grp_name = name + ' ' + argv[i+1]
                    for k,_ in self.commands.items():
                        if grp_name == k:
                            argv = argv[(i + 1 + 1):]
                            name = grp_name
                            got_flag = True
                            break
                    if got_flag:
                        break
                argv = argv[i + 1:]
                break
        if not name:
            glob = argv
            name = 'help'
            argv = []
        gopts, _gargs = global_options.parse_args(glob)
        return (name, gopts, argv)

    def _Run(self, name, gopts, argv):
        result = 0
        try:
            cmd = self.commands[name]

        except KeyError:
            if not self.conf.init:
                put_string("aos: the workspace is not initialized.", file=sys.stderr)
                if self.conf.need_auto_init:
                    put_string("note: auto initialize workspace in %s.\n" % self.conf.yoc_path)
                    cur_dir = os.getcwd()
                    os.chdir(self.conf.yoc_path)
                    init_cmd = all_commands['init']
                    init_argv = []
                    copts, cargs = init_cmd.OptionParser.parse_args(init_argv)
                    result = init_cmd.Execute(copts, cargs)
                    os.chdir(cur_dir)
                    self.conf.need_auto_init = False
                    return __auto_init__
                else:
                    put_string("\nnote: please enter into the workspace directory.")
                    put_string("      and run 'aos init' in the workspace root directory.")
                    return 1
            else:
                put_string("aos: '%s' is not a aos command.  See 'aos help'." % name, file=sys.stderr)
                check_remote_version(__version__)
                return 1

        try:
            copts, cargs = cmd.OptionParser.parse_args(argv)
            copts = cmd.ReadEnvironmentOptions(copts)
        except Exception as e:
            put_string('error: in `%s`: %s' % (' '.join([name] + argv), str(e)),
                       file=sys.stderr)
            put_string('error: manifest missing or unreadable -- please run init',
                       file=sys.stderr)
            return 1
        try:
            cmd.ValidateOptions(copts, cargs)
            result = cmd.Execute(copts, cargs)
        except Exception as e:
            put_string("AosCommand error:", e)
            check_remote_version(__version__)
            result = 1
            pass
        return result

    def Execute(self, argv):
        name, gopts, argv = self._ParseArgs(argv)

        return self._Run(name, gopts, argv)


def main():
    if len(sys.argv) == 2:
        if sys.argv[1] == '-V' or sys.argv[1] == '--version':
            put_string("aos-tools " + __version__ + "\n")
            path = which("aos-tools")
            put_string("script path is: " + path)
            put_string("pip package path is: " + os.path.dirname(os.path.realpath(__file__)))
            check_remote_version(__version__)
            exit(0)
    cmd = YocCommand()
    result = cmd.Execute(sys.argv[1:])
    # rerun it after auto initialze the workspace
    if result == __auto_init__:
        cmd = YocCommand()
        result = cmd.Execute(sys.argv[1:])
    exit(result)

def cct_main():
    try:
        cmd = all_commands['cct']
        parser = cmd.OptionParser
        parser.set_usage(cmd.helpUsage.strip().replace('%prog', cmd.NAME))
        copts, cargs = parser.parse_args(sys.argv[1:])
        copts = cmd.ReadEnvironmentOptions(copts)
    except Exception as e:
        put_string('error: manifest missing or unreadable -- please run init',
                    file=sys.stderr)
        return 1
    try:
        cmd.ValidateOptions(copts, cargs)
        cmd.Execute(copts, cargs)
    except Exception as e:
        put_string("AosCommand error:", e)
        pass



# -*- coding:utf-8 -*-
#
# Copyright (C) 2019-2020 Alibaba Group Holding Limited


from __future__ import print_function

from aostools import *
import os
import shutil

class Export(Command):
    common = True
    helpSummary = "Export component to dest directory"
    helpUsage = """
%prog <component> [<component> <component> ...] <dest path>
"""
    helpDescription = """
Export component to dest directory
"""

    def Execute(self, opt, args):
        if len(args) < 2:
            self.Usage()

        yoc = YoC()
        componentList = args[:-1]
        validComponentList = []
        destPath = args[-1]

        for component in yoc.components:
            if component.name in componentList:
                component.load_package()
                depend = yoc.check_depend(component)
                for d in depend:
                    self.component_export(d, destPath)
                self.component_export(component, destPath)
                validComponentList.append(component.name)
        if validComponentList:
            yoc.conf.save(os.path.join(destPath, '.aos'))
        invalidComponentList = set(componentList).difference(set(validComponentList))
        if invalidComponentList:
            put_string("These components [%s] are not existed!" % (", ").join(invalidComponentList))
        return 0


    def component_export(self, component, path):
        p = get_cmpt_path_by_type(component.type)
        if p:
            dest = os.path.join(path, p, component.name)
            put_string("Export `%s` to %s." % (component.name, dest))
            try:
                if os.path.exists(dest):
                    rmtree_enhanced(dest)
                shutil.copytree(component.path, dest)
            except Exception as ex:
                put_string(str(ex))
                pass



# -*- coding:utf-8 -*-
#
# Copyright (C) 2019-2020 Alibaba Group Holding Limited


from __future__ import print_function

from aostools import *
import codecs
import time
import multiprocessing;


class Make(Command):
    common = True
    helpSummary = "Make aos program"
    helpUsage = """
%prog [option] [<subcommand> ...]
"""
    helpDescription = """
Make aos program.\n"
"\nPlease run 'aos make --help' to explore more ...
"""

    def _Options(self, p):
        p.add_option('-b', '--board',
                     dest='board', action='store',
                     help='select board to build')
        p.add_option('-c', '--config',
                     dest='config', action='store',
                     help='select solution and board')


    def Execute(self, opt, args):
        result = 0
        cmds = []
        params = []
        for arg in args:
            if "=" in arg:
                params.append(arg)
            else:
                cmds.append(arg)
        
        params_str = " ".join(params)
        # get workspace root directory
        conf = Configure()
        cur_dir = os.getcwd()
        # enter into solution directory if in the workspace root directory
        if cur_dir == conf.yoc_path:
            solution, board = getSolutionAndBoard(conf.yoc_path)
            if solution:
                new_dir = os.path.join(cur_dir, "solutions", solution)
                if os.path.isdir(new_dir):
                    os.chdir(new_dir)
                    put_string("change dir to %s." % new_dir)
            if board:
                # read board from .config, and set to opt.board. Like aos make -b board
                opt.board = board

        ext_scons_existed = False
        if get_host_os() == 'Win32':
            ext_scons = which('scons.exe')
            if ext_scons:
                ext_scons_existed = True
                put_string("Found incompatible version of SCons at %s." % ext_scons)
                put_string("It is better to uninstall it to avoid unexpected issues.\n")

        num = multiprocessing.cpu_count()
        multi_thread = "-j" + str(num)
        count = len(cmds)
        if count == 0:
            starttime = time.time()
            if opt.board:
                print("save aos make params !!!")
                self.saveSolutionAndBoardParams(conf.yoc_path, opt.board)
                if ext_scons_existed:
                    if params_str.find("V=1") >= 0:
                        result = os.system("scons --verbose --board=%s -j%d  %s" % (opt.board, num, params_str)) >> 8
                    else:
                        result = os.system("scons --board=%s -j%d %s" % (opt.board, num, params_str)) >> 8
                else:
                    result = os.system("make BOARD=%s MULTITHREADS=%s %s" % (opt.board, multi_thread, params_str)) >> 8
            else:
                if ext_scons_existed:
                    if params_str.find("V=1") >= 0:
                        result = os.system("scons --verbose -j%d %s" % (num, params_str)) >> 8
                    else:
                        result = os.system("scons -j%d %s" % (num, params_str)) >> 8
                else:
                    result = os.system("make MULTITHREADS=%s %s" % (multi_thread, params_str)) >> 8
            maketime = time.time() - starttime
            m, s = divmod(maketime, 60)
            put_string("Build takes %s Minutes %s Seconds." % (int(m), int(s)))
        for cmd in cmds:
            if cmd == "clean":
                if opt.board:
                    if ext_scons_existed:
                        result = os.system("scons -c --board=%s" % opt.board) >> 8
                    else:
                        result = os.system("make clean BOARD=%s" % opt.board) >> 8
                else:
                    if ext_scons_existed:
                        result = os.system("scons -c") >> 8
                    else:
                        result = os.system("make clean") >> 8
                break
            elif cmd == "distclean":
                self.clearSolutionAndBoard(conf.yoc_path)
                if opt.board:
                    if ext_scons_existed:
                        result = os.system("scons -c --board=%s" % opt.board) >> 8
                    else:
                        result = os.system("make clean BOARD=%s" % opt.board) >> 8
                else:
                    if ext_scons_existed:
                        result = os.system("scons -c") >> 8
                    else:
                        result = os.system("make clean") >> 8
                break
            elif "@" in cmd:
                self.saveSolutionAndBoard(conf.yoc_path, cmd)
            else:
                result = os.system("make %s %s" % (cmd, params_str)) >> 8
        return result

    def saveSolutionAndBoardParams(self, yoc_path, app_board):
        config_file = os.path.join(yoc_path, ".config")
        if not os.path.isfile(os.path.join(yoc_path, ".aos")):
            put_string("Please excute \"aos init\" in the workspace root directory first.")
            exit(-1)
        else:
            # check params for save
            print("get current path")
            cur_dir = os.getcwd()
            filename = os.path.join(cur_dir, 'package.yaml')
            solutionName = os.path.basename(cur_dir)
            print(filename)
            print(solutionName)

            if os.path.isfile(filename):
                pack = Package(filename)
                for d in pack.depends:
                    if pack.hw_info.board_name in d:
                        pack.depends.remove(d)
                    l = str(d).replace("}","").replace("'","").split(":")
                depend = {app_board: l[1]}
                pack.depends.append(depend)
                pack.hw_info.board_name = app_board
                pack.save(filename)
                # save solution and board in .config file
                with codecs.open(config_file, 'w', 'UTF-8') as f:
                    f.write("{}: {}\n".format("solution", solutionName))
                    f.write("{}: {}\n".format("board", app_board))
                    put_string("configuration %s written to %s." % (app_board, config_file))


    def saveSolutionAndBoard(self, yoc_path, app_board):
        config_file = os.path.join(yoc_path, ".config")
        if not os.path.isfile(os.path.join(yoc_path, ".aos")):
            put_string("Please excute \"aos init\" in the workspace root directory first.")
            exit(-1)
        else:
            v = app_board.split("@")
            if len(v) != 2:
                put_string("Invalid format. Please config it like aos make solution@board -c config")
                exit(-1)

            # update package.yaml of solution: set the specified board as default one
            filename = os.path.join(yoc_path, "solutions", v[0], 'package.yaml')

            if os.path.isfile(filename):
                pack = Package(filename)
                for d in pack.depends:
                    if pack.hw_info.board_name in d:
                        pack.depends.remove(d)
                    l = str(d).replace("}","").replace("'","").split(":")
                depend = {v[1]: l[1]}
                pack.depends.append(depend)
                pack.hw_info.board_name = v[1]
                pack.save(filename)
                # save solution and board in .config file
                with codecs.open(config_file, 'w', 'UTF-8') as f:
                    f.write("{}: {}\n".format("solution", v[0]))
                    f.write("{}: {}\n".format("board", v[1]))
                    put_string("configuration %s written to %s." % (app_board, config_file))
            else:
                put_string("solution %s is not existed." % v[0])

    def clearSolutionAndBoard(self, yoc_path):
        config_file = os.path.join(yoc_path, ".config")
        if os.path.isfile(config_file):
            os.remove(config_file)
        if os.path.isfile('.config_burn'):
            os.remove('.config_burn')
        if os.path.isfile('.config_monitor'):
            os.remove('.config_monitor')



# -*- coding:utf-8 -*-
#
# Copyright (C) 2019-2020 Alibaba Group Holding Limited


from __future__ import print_function

import os
from aostools import *

class Init(Command):
    common = True
    helpSummary = "Initialize aos workspace in the current directory"
    helpUsage = """
%prog [repo] [option] <branch>
"""
    helpDescription = """
Initialize aos workspace in the current directory.
"""
    def _Options(self, p):
        p.add_option('-b', '--branch',
                     dest='branch', action='store', type='string',
                     help='the manifest repo branch')

    def Execute(self, opt, args):
        conf = Configure()
        if conf.init:
            put_string("[Error]: Workspace is initialized already.")
            put_string("The workspace is `%s`" % conf.yoc_path)
            return 1
        if os.path.isfile("package.yaml"):
            put_string("[Error]: Please intialize it in the workspace root directory.")
            return 1
        if os.path.isdir(os.path.join(conf.yoc_path, '.aos')):
            put_string("[Error]: It is an invalid directory `%s`." % conf.yoc_path)
            return 1
        if os.path.isfile(os.path.join("build", "Makefile")) and os.path.isfile(os.path.join("build", "Config.in")):
            put_string("[Error]: There are `build/Makefile` and `build/Config.in` in the current directory.")
            put_string("It can not intialize the workspace in the AliOS Things V3.1(or older version) directory.")
            return 1
        conf.gitee_token = "32e702f14950e51e3a230e7ca8a60f10"
        conf.occ_host = "occ.t-head.cn"
        conf.repo = "https://gitee.com/alios-things/manifest.git"
        conf.branch = "master"
        conf.need_auto_init = False
        # try:
        #     urlretrieve('https://yoctools.oss-cn-beijing.aliyuncs.com/yoc_new', '.aos')
        # except:
        #     pass
        if len(args) > 0:
            # conf.load(os.path.join(conf.yoc_path, '.aos'))
            repourl = args[0]
            repobr = 'master'
            if opt.branch:
                repobr = opt.branch
            conf.branch = repobr
            conf.repo = repourl
            conf.save()
        if not os.path.isfile('.aos'):
            conf.save()
        return 0



# -*- coding:utf-8 -*-
#
# Copyright (C) 2019-2020 Alibaba Group Holding Limited


from __future__ import print_function

from aostools import *


class Exportall(Command):
    common = True
    helpSummary = "Export file to aos_sdk dir"
    helpUsage = """
%prog [option]
"""
    helpDescription = """
Display the detailed compilation information of the current solution.
"""

    def _Options(self, p):
        p.add_option('-b', '--board',
                     dest='board_name', action='store', type='str', default=None,
                     help='show configuration of selected board')


    def Execute(self, opt, args):
        yoc = YoC()
        solution = yoc.getSolution(opt.board_name)
        if solution == None:
            put_string("The current directory is not a solution!")
            exit(-1)
        solution.install() 
        return 0



# -*- coding:utf-8 -*-
#
# Copyright (C) 2019-2020 Alibaba Group Holding Limited


from __future__ import print_function

from aostools import *


class Variable(Command):
    common = True
    helpSummary = "Display variables for the current solution"
    helpUsage = """
%prog [option] [<variable>...]
"""
    helpDescription = """
Display variables for the current solution
"""
    def _Options(self, p):
        p.add_option('-b', '--board',
                     dest='board_name', action='store', type='str', default=None,
                     help='show configuration of selected board')

    def Execute(self, opt, args):
        yoc = YoC()
        solution = yoc.getSolution(opt.board_name)
        if solution == None:
            put_string("The current directory is not a solution!")
            exit(-1)
        if len(args) == 0:
            for k, v in solution.variables.items():
                put_string("%-10s = %s" % (k, v))
        else:
            for arg in args:
                var = solution.variables.get(arg)
                put_string(var)
        return 0



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from distutils.command.build_ext import build_ext as _build_ext
from distutils.core import Extension
import os
import sys
import platform
import site
from codecs import open  # To use a consistent encoding

# Always prefer setuptools over distutils
from setuptools import setup, find_packages

def read(rel_path):
    here = os.path.abspath(os.path.dirname(__file__))
    with open(os.path.join(here, rel_path), 'r') as fp:
        return fp.read()

def get_version(rel_path):
    for line in read(rel_path).splitlines():
        if line.startswith('__version__'):
            # __version__ = "0.9"
            delim = '"' if '"' in line else "'"
            return line.split(delim)[1]
    else:
        raise RuntimeError("Unable to find version string.")

APP_NAME = 'aos-tools'

settings = dict()

thead_tools = []
if platform.system() == "Linux":
    thead_tools=[
        ('bin', ['aostools/build/product64']),
        ('bin', ['aostools/build/product32']),
        ('bin', ['aostools/build/gen_ldfile.sh']),
        ('/usr/local/bin', ['aostools/build/product64']),
        ('/usr/local/bin', ['aostools/build/product32']),
        ('/usr/local/bin', ['aostools/build/gen_ldfile.sh']),
        ('/usr/local/lib/aostools/script', [
            'aostools/script/aft_build.sh',
            'aostools/script/cdkgdbinit',
            'aostools/script/flash.init',
            'aostools/script/flash_prog.init',
            'aostools/script/pre_build.sh',
        ])]

requirements=[
        'scons>=3.0.0, <4.0.0',
        'requests_toolbelt',
        'threadpool',
        'smmap',
        'configparser==4.0.2',
        'pyyaml',
        'pyserial',
        'glob2',
        'chardet']
if platform.system() == "Windows":
    requirements.append('pywin32')

settings.update(
    name=APP_NAME,
    version=get_version("aostools/cmd.py"),
    description='aos tools',
    author='Zhuzhg',
    author_email='zzg@ifnfn.com',
    packages=find_packages(),
    # packages = ['aostools', 'git', 'gitdb', 'yaml'],
    install_requires=requirements,

    license='BSD',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    data_files=thead_tools,
    entry_points={
        'console_scripts': [
            'aos = aostools.cmd:main',
            'aos-tools = aostools.cmd:main',
            'cct = aostools.cmd:cct_main'
        ],
    },

    python_requires='>=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*, !=3.4.*',
)


setup(**settings)

if platform.system() == "Linux":
    if os.getuid() != 0:
        install_path = os.path.join(site.USER_BASE, 'bin', 'product')
    else:
        install_path = os.path.join(sys.prefix, 'bin', 'product')

    architecture = platform.architecture()

    if architecture[0] == '64bit':
        product = install_path + '64'
    else:
        product = install_path + '32'

    print("create link from %s to %s" %(product, install_path))
    try:
        os.symlink(product, install_path)
    except:
        pass



# -*- coding:utf-8 -*-
#
# Copyright (C) 2019-2020 Alibaba Group Holding Limited


from __future__ import print_function

from aostools import *


class Addkey(Command):
    common = True
    helpSummary = "Add your RSA public key into repo"
    helpUsage = """
%prog
"""
    helpDescription = """
Add your RSA public key into repo.
"""

    def Execute(self, opt, args):
        conf = Configure()
        repo = RepoGitee(conf.gitee_token, conf.group)
        repo.add_public_key()

